@@ -20,16 +20,13 @@
 
 /* Version parsed out into numeric values */
 #define PY_MAJOR_VERSION	2
-#define PY_MINOR_VERSION	1
-#define PY_MICRO_VERSION	2
+#define PY_MINOR_VERSION	3
+#define PY_MICRO_VERSION	7
 #define PY_RELEASE_LEVEL	PY_RELEASE_LEVEL_FINAL
 #define PY_RELEASE_SERIAL	0
 
 /* Version as a string */
-#define PY_VERSION		"2.1.2"
-
-/* Historic */
-#define PATCHLEVEL		"2.1.2"
+#define PY_VERSION		"2.3.7"
 
 /* Version as a single 4-byte hex number, e.g. 0x010502B2 == 1.5.2b2.
    Use this for numeric comparisons, e.g. #if PY_VERSION_HEX >= ... */
