@@ -12,8 +12,13 @@
 #include <signal.h>
 #include <errno.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h> /* For isatty() */
+#if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
+/* GNU readline() mistakenly sets the LC_CTYPE locale.
+ * This is evil.  Only the user or the app's main() should do this!
+ * We must save and restore the locale around the rl_initialize() call.
+ */
+#define SAVE_LOCALE
+#include <locale.h>
 #endif
 
 /* GNU readline definitions */
@@ -22,13 +27,10 @@
 #include <readline/history.h>
 
 #ifdef HAVE_RL_COMPLETION_MATCHES
-#define completion_matches(x, y) rl_completion_matches((x), ((rl_compentry_func_t *)(y)))
+#define completion_matches(x, y) \
+	rl_completion_matches((x), ((rl_compentry_func_t *)(y)))
 #endif
 
-/* Pointers needed from outside (but not declared in a header file). */
-extern DL_IMPORT(int) (*PyOS_InputHook)(void);
-extern DL_IMPORT(char) *(*PyOS_ReadlineFunctionPointer)(char *);
-
 
 /* Exported function to send one line to readline's init file parser */
 
@@ -50,10 +52,9 @@ parse_and_bind(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char doc_parse_and_bind[] = "\
-parse_and_bind(string) -> None\n\
-Parse and execute single line of a readline init file.\
-";
+PyDoc_STRVAR(doc_parse_and_bind,
+"parse_and_bind(string) -> None\n\
+Parse and execute single line of a readline init file.");
 
 
 /* Exported function to parse a readline init file */
@@ -71,11 +72,10 @@ read_init_file(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char doc_read_init_file[] = "\
-read_init_file([filename]) -> None\n\
+PyDoc_STRVAR(doc_read_init_file,
+"read_init_file([filename]) -> None\n\
 Parse a readline initialization file.\n\
-The default filename is the last filename used.\
-";
+The default filename is the last filename used.");
 
 
 /* Exported function to load a readline history file */
@@ -94,11 +94,10 @@ read_history_file(PyObject *self, PyObject *args)
 }
 
 static int history_length = -1; /* do not truncate history by default */
-static char doc_read_history_file[] = "\
-read_history_file([filename]) -> None\n\
+PyDoc_STRVAR(doc_read_history_file,
+"read_history_file([filename]) -> None\n\
 Load a readline history file.\n\
-The default filename is ~/.history.\
-";
+The default filename is ~/.history.");
 
 
 /* Exported function to save a readline history file */
@@ -118,50 +117,51 @@ write_history_file(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char doc_write_history_file[] = "\
-write_history_file([filename]) -> None\n\
+PyDoc_STRVAR(doc_write_history_file,
+"write_history_file([filename]) -> None\n\
 Save a readline history file.\n\
-The default filename is ~/.history.\
-";
+The default filename is ~/.history.");
 
 
-static char set_history_length_doc[] = "\
-set_history_length(length) -> None\n\
-set the maximal number of items which will be written to\n\
-the history file. A negative length is used to inhibit\n\
-history truncation.\n\
-";
+/* Set history length */
 
 static PyObject*
 set_history_length(PyObject *self, PyObject *args)
 {
-    int length = history_length;
-    if (!PyArg_ParseTuple(args, "i:set_history_length", &length))
-	return NULL;
-    history_length = length;
-    Py_INCREF(Py_None);
-    return Py_None;
+	int length = history_length;
+	if (!PyArg_ParseTuple(args, "i:set_history_length", &length))
+		return NULL;
+	history_length = length;
+	Py_INCREF(Py_None);
+	return Py_None;
 }
 
+PyDoc_STRVAR(set_history_length_doc,
+"set_history_length(length) -> None\n\
+set the maximal number of items which will be written to\n\
+the history file. A negative length is used to inhibit\n\
+history truncation.");
 
 
-static char get_history_length_doc[] = "\
-get_history_length() -> int\n\
-return the current history length value.\n\
-";
+/* Get history length */
 
 static PyObject*
-get_history_length(PyObject *self, PyObject *args)
+get_history_length(PyObject *self, PyObject *noarg)
 {
-	if (!PyArg_ParseTuple(args, ":get_history_length"))
-		return NULL;
-	return Py_BuildValue("i", history_length);
+	return PyInt_FromLong(history_length);
 }
 
+PyDoc_STRVAR(get_history_length_doc,
+"get_history_length() -> int\n\
+return the maximum number of items that will be written to\n\
+the history file.");
+
+
 /* Generic hook function setter */
 
 static PyObject *
-set_hook(const char * funcname, PyObject **hook_var, PyThreadState **tstate, PyObject *args)
+set_hook(const char *funcname, PyObject **hook_var,
+	 PyThreadState **tstate, PyObject *args)
 {
 	PyObject *function = Py_None;
 	char buf[80];
@@ -191,6 +191,7 @@ set_hook(const char * funcname, PyObject **hook_var, PyThreadState **tstate, PyO
 	return Py_None;
 }
 
+
 /* Exported functions to specify hook functions in Python */
 
 static PyObject *startup_hook = NULL;
@@ -204,32 +205,38 @@ static PyThreadState *pre_input_hook_tstate = NULL;
 static PyObject *
 set_startup_hook(PyObject *self, PyObject *args)
 {
-	return set_hook("startup_hook", &startup_hook, &startup_hook_tstate, args);
+	return set_hook("startup_hook", &startup_hook,
+			&startup_hook_tstate, args);
 }
 
-static char doc_set_startup_hook[] = "\
-set_startup_hook([function]) -> None\n\
+PyDoc_STRVAR(doc_set_startup_hook,
+"set_startup_hook([function]) -> None\n\
 Set or remove the startup_hook function.\n\
 The function is called with no arguments just\n\
-before readline prints the first prompt.\n\
-";
+before readline prints the first prompt.");
+
 
 #ifdef HAVE_RL_PRE_INPUT_HOOK
+
+/* Set pre-input hook */
+
 static PyObject *
 set_pre_input_hook(PyObject *self, PyObject *args)
 {
-	return set_hook("pre_input_hook", &pre_input_hook,  &pre_input_hook_tstate, args);
+	return set_hook("pre_input_hook", &pre_input_hook,
+			&pre_input_hook_tstate, args);
 }
 
-static char doc_set_pre_input_hook[] = "\
-set_pre_input_hook([function]) -> None\n\
+PyDoc_STRVAR(doc_set_pre_input_hook,
+"set_pre_input_hook([function]) -> None\n\
 Set or remove the pre_input_hook function.\n\
 The function is called with no arguments after the first prompt\n\
 has been printed and just before readline starts reading input\n\
-characters.\n\
-";
+characters.");
+
 #endif
 
+
 /* Exported function to specify a word completer in Python */
 
 static PyObject *completer = NULL;
@@ -238,38 +245,36 @@ static PyThreadState *completer_tstate = NULL;
 static PyObject *begidx = NULL;
 static PyObject *endidx = NULL;
 
-/* get the beginning index for the scope of the tab-completion */
+
+/* Get the beginning index for the scope of the tab-completion */
+
 static PyObject *
-get_begidx(PyObject *self, PyObject *args)
+get_begidx(PyObject *self, PyObject *noarg)
 {
-	if(!PyArg_NoArgs(args)) {
-		return NULL;
-	} 
 	Py_INCREF(begidx);
 	return begidx;
 }
 
-static char doc_get_begidx[] = "\
-get_begidx() -> int\n\
-get the beginning index of the readline tab-completion scope";
+PyDoc_STRVAR(doc_get_begidx,
+"get_begidx() -> int\n\
+get the beginning index of the readline tab-completion scope");
+
+
+/* Get the ending index for the scope of the tab-completion */
 
-/* get the ending index for the scope of the tab-completion */
 static PyObject *
-get_endidx(PyObject *self, PyObject *args)
+get_endidx(PyObject *self, PyObject *noarg)
 {
- 	if(!PyArg_NoArgs(args)) {
-		return NULL;
-	} 
 	Py_INCREF(endidx);
 	return endidx;
 }
 
-static char doc_get_endidx[] = "\
-get_endidx() -> int\n\
-get the ending index of the readline tab-completion scope";
+PyDoc_STRVAR(doc_get_endidx,
+"get_endidx() -> int\n\
+get the ending index of the readline tab-completion scope");
 
 
-/* set the tab-completion word-delimiters that readline uses */
+/* Set the tab-completion word-delimiters that readline uses */
 
 static PyObject *
 set_completer_delims(PyObject *self, PyObject *args)
@@ -279,15 +284,18 @@ set_completer_delims(PyObject *self, PyObject *args)
 	if(!PyArg_ParseTuple(args, "s:set_completer_delims", &break_chars)) {
 		return NULL;
 	}
-	free(rl_completer_word_break_characters);
+	free((void*)rl_completer_word_break_characters);
 	rl_completer_word_break_characters = strdup(break_chars);
 	Py_INCREF(Py_None);
 	return Py_None;
 }
 
-static char doc_set_completer_delims[] = "\
-set_completer_delims(string) -> None\n\
-set the readline word delimiters for tab-completion";
+PyDoc_STRVAR(doc_set_completer_delims,
+"set_completer_delims(string) -> None\n\
+set the readline word delimiters for tab-completion");
+
+
+/* Add a line to the history buffer */
 
 static PyObject *
 py_add_history(PyObject *self, PyObject *args)
@@ -302,25 +310,25 @@ py_add_history(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
-static char doc_add_history[] = "\
-add_history(string) -> None\n\
-add a line to the history buffer";
+PyDoc_STRVAR(doc_add_history,
+"add_history(string) -> None\n\
+add a line to the history buffer");
 
 
-/* get the tab-completion word-delimiters that readline uses */
+/* Get the tab-completion word-delimiters that readline uses */
 
 static PyObject *
-get_completer_delims(PyObject *self, PyObject *args)
+get_completer_delims(PyObject *self, PyObject *noarg)
 {
-	if(!PyArg_NoArgs(args)) {
-		return NULL;
-	}
 	return PyString_FromString(rl_completer_word_break_characters);
 }
-	
-static char doc_get_completer_delims[] = "\
-get_completer_delims() -> string\n\
-get the readline word delimiters for tab-completion";
+
+PyDoc_STRVAR(doc_get_completer_delims,
+"get_completer_delims() -> string\n\
+get the readline word delimiters for tab-completion");
+
+
+/* Set the completer function */
 
 static PyObject *
 set_completer(PyObject *self, PyObject *args)
@@ -328,28 +336,81 @@ set_completer(PyObject *self, PyObject *args)
 	return set_hook("completer", &completer, &completer_tstate, args);
 }
 
-static char doc_set_completer[] = "\
-set_completer([function]) -> None\n\
+PyDoc_STRVAR(doc_set_completer,
+"set_completer([function]) -> None\n\
 Set or remove the completer function.\n\
 The function is called as function(text, state),\n\
 for state in 0, 1, 2, ..., until it returns a non-string.\n\
-It should return the next possible completion starting with 'text'.\
-";
+It should return the next possible completion starting with 'text'.");
+
+
+static PyObject *
+get_completer(PyObject *self, PyObject *noargs)
+{
+	if (completer == NULL) {
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
+	Py_INCREF(completer);
+	return completer;
+}
+
+PyDoc_STRVAR(doc_get_completer,
+"get_completer() -> function\n\
+\n\
+Returns current completer function.");
+
+/* Exported function to get any element of history */
+
+static PyObject *
+get_history_item(PyObject *self, PyObject *args)
+{
+	int idx = 0;
+	HIST_ENTRY *hist_ent;
+
+	if (!PyArg_ParseTuple(args, "i:index", &idx))
+		return NULL;
+	if ((hist_ent = history_get(idx)))
+		return PyString_FromString(hist_ent->line);
+	else {
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
+}
+
+PyDoc_STRVAR(doc_get_history_item,
+"get_history_item() -> string\n\
+return the current contents of history item at index.");
+
+
+/* Exported function to get current length of history */
+
+static PyObject *
+get_current_history_length(PyObject *self, PyObject *noarg)
+{
+	HISTORY_STATE *hist_st;
+
+	hist_st = history_get_history_state();
+	return PyInt_FromLong(hist_st ? (long) hist_st->length : (long) 0);
+}
+
+PyDoc_STRVAR(doc_get_current_history_length,
+"get_current_history_length() -> integer\n\
+return the current (not the maximum) length of history.");
+
 
 /* Exported function to read the current line buffer */
 
 static PyObject *
-get_line_buffer(PyObject *self, PyObject *args)
+get_line_buffer(PyObject *self, PyObject *noarg)
 {
-	if (!PyArg_NoArgs(args))
-		return NULL;
 	return PyString_FromString(rl_line_buffer);
 }
 
-static char doc_get_line_buffer[] = "\
-get_line_buffer() -> string\n\
-return the current contents of the line buffer.\
-";
+PyDoc_STRVAR(doc_get_line_buffer,
+"get_line_buffer() -> string\n\
+return the current contents of the line buffer.");
+
 
 /* Exported function to insert text into the line buffer */
 
@@ -364,11 +425,25 @@ insert_text(PyObject *self, PyObject *args)
 	return Py_None;
 }
 
+PyDoc_STRVAR(doc_insert_text,
+"insert_text(string) -> None\n\
+Insert text into the command line.");
 
-static char doc_insert_text[] = "\
-insert_text(string) -> None\n\
-Insert text into the command line.\
-";
+
+/* Redisplay the line buffer */
+
+static PyObject *
+redisplay(PyObject *self, PyObject *noarg)
+{
+	rl_redisplay();
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
+PyDoc_STRVAR(doc_redisplay,
+"redisplay() -> None\n\
+Change what's displayed on the screen to reflect the current\n\
+contents of the line buffer.");
 
 
 /* Table of functions exported by the module */
@@ -376,54 +451,60 @@ Insert text into the command line.\
 static struct PyMethodDef readline_methods[] =
 {
 	{"parse_and_bind", parse_and_bind, METH_VARARGS, doc_parse_and_bind},
-	{"get_line_buffer", get_line_buffer, 
-	 METH_OLDARGS, doc_get_line_buffer},
+	{"get_line_buffer", get_line_buffer, METH_NOARGS, doc_get_line_buffer},
 	{"insert_text", insert_text, METH_VARARGS, doc_insert_text},
+	{"redisplay", redisplay, METH_NOARGS, doc_redisplay},
 	{"read_init_file", read_init_file, METH_VARARGS, doc_read_init_file},
-	{"read_history_file", read_history_file, 
+	{"read_history_file", read_history_file,
 	 METH_VARARGS, doc_read_history_file},
-	{"write_history_file", write_history_file, 
+	{"write_history_file", write_history_file,
 	 METH_VARARGS, doc_write_history_file},
- 	{"set_history_length", set_history_length, 
+	{"get_history_item", get_history_item,
+	 METH_VARARGS, doc_get_history_item},
+	{"get_current_history_length", (PyCFunction)get_current_history_length,
+	 METH_NOARGS, doc_get_current_history_length},
+ 	{"set_history_length", set_history_length,
 	 METH_VARARGS, set_history_length_doc},
- 	{"get_history_length", get_history_length, 
-	 METH_VARARGS, get_history_length_doc},
+ 	{"get_history_length", get_history_length,
+	 METH_NOARGS, get_history_length_doc},
 	{"set_completer", set_completer, METH_VARARGS, doc_set_completer},
-	{"get_begidx", get_begidx, METH_OLDARGS, doc_get_begidx},
-	{"get_endidx", get_endidx, METH_OLDARGS, doc_get_endidx},
+	{"get_completer", get_completer, METH_NOARGS, doc_get_completer},
+	{"get_begidx", get_begidx, METH_NOARGS, doc_get_begidx},
+	{"get_endidx", get_endidx, METH_NOARGS, doc_get_endidx},
 
-	{"set_completer_delims", set_completer_delims, 
+	{"set_completer_delims", set_completer_delims,
 	 METH_VARARGS, doc_set_completer_delims},
 	{"add_history", py_add_history, METH_VARARGS, doc_add_history},
-	{"get_completer_delims", get_completer_delims, 
-	 METH_OLDARGS, doc_get_completer_delims},
-	
-	{"set_startup_hook", set_startup_hook, METH_VARARGS, doc_set_startup_hook},
+	{"get_completer_delims", get_completer_delims,
+	 METH_NOARGS, doc_get_completer_delims},
+
+	{"set_startup_hook", set_startup_hook,
+	 METH_VARARGS, doc_set_startup_hook},
 #ifdef HAVE_RL_PRE_INPUT_HOOK
-	{"set_pre_input_hook", set_pre_input_hook, METH_VARARGS, doc_set_pre_input_hook},
+	{"set_pre_input_hook", set_pre_input_hook,
+	 METH_VARARGS, doc_set_pre_input_hook},
 #endif
 	{0, 0}
 };
 
+
 /* C function to call the Python hooks. */
 
 static int
-on_hook(PyObject *func, PyThreadState *tstate)
+on_hook(PyObject *func, PyThreadState **tstate)
 {
 	int result = 0;
 	if (func != NULL) {
 		PyObject *r;
-		PyThreadState *save_tstate;
 		/* Note that readline is called with the interpreter
 		   lock released! */
-		save_tstate = PyThreadState_Swap(NULL);
-		PyEval_RestoreThread(tstate);
+		PyEval_RestoreThread(*tstate);
 		r = PyObject_CallFunction(func, NULL);
 		if (r == NULL)
 			goto error;
-		if (r == Py_None) 
+		if (r == Py_None)
 			result = 0;
-		else 
+		else
 			result = PyInt_AsLong(r);
 		Py_DECREF(r);
 		goto done;
@@ -431,8 +512,7 @@ on_hook(PyObject *func, PyThreadState *tstate)
 		PyErr_Clear();
 		Py_XDECREF(r);
 	  done:
-		PyEval_SaveThread();
-		PyThreadState_Swap(save_tstate);
+		*tstate = PyEval_SaveThread();
 	}
 	return result;
 }
@@ -440,17 +520,18 @@ on_hook(PyObject *func, PyThreadState *tstate)
 static int
 on_startup_hook(void)
 {
-	return on_hook(startup_hook, startup_hook_tstate);
+	return on_hook(startup_hook, &startup_hook_tstate);
 }
 
 #ifdef HAVE_RL_PRE_INPUT_HOOK
 static int
 on_pre_input_hook(void)
 {
-	return on_hook(pre_input_hook, pre_input_hook_tstate);
+	return on_hook(pre_input_hook, &pre_input_hook_tstate);
 }
 #endif
 
+
 /* C function to call the Python completer. */
 
 static char *
@@ -459,11 +540,12 @@ on_completion(char *text, int state)
 	char *result = NULL;
 	if (completer != NULL) {
 		PyObject *r;
-		PyThreadState *save_tstate;
 		/* Note that readline is called with the interpreter
 		   lock released! */
-		save_tstate = PyThreadState_Swap(NULL);
 		PyEval_RestoreThread(completer_tstate);
+		/* Don't use the default filename completion if we
+		 * have a custom completion function... */
+		rl_attempted_completion_over = 1;
 		r = PyObject_CallFunction(completer, "si", text, state);
 		if (r == NULL)
 			goto error;
@@ -482,17 +564,16 @@ on_completion(char *text, int state)
 		PyErr_Clear();
 		Py_XDECREF(r);
 	  done:
-		PyEval_SaveThread();
-		PyThreadState_Swap(save_tstate);
+		completer_tstate = PyEval_SaveThread();
 	}
 	return result;
 }
 
 
-/* a more flexible constructor that saves the "begidx" and "endidx"
+/* A more flexible constructor that saves the "begidx" and "endidx"
  * before calling the normal completer */
 
-char **
+static char **
 flex_complete(char *text, int start, int end)
 {
 	Py_XDECREF(begidx);
@@ -502,12 +583,25 @@ flex_complete(char *text, int start, int end)
 	return completion_matches(text, *on_completion);
 }
 
+
 /* Helper to initialize GNU readline properly. */
 
 static void
 setup_readline(void)
 {
+#ifdef SAVE_LOCALE
+	char *saved_locale = strdup(setlocale(LC_CTYPE, NULL));
+	if (!saved_locale)
+		Py_FatalError("not enough memory to save locale");
+#endif
+
+	using_history();
+
 	rl_readline_name = "python";
+#if defined(PYOS_OS2) && defined(PYCC_GCC)
+	/* Allow $if term= in .inputrc to work */
+	rl_terminal_name = getenv("TERM");
+#endif
 	/* Force rebind of TAB to insert-tab */
 	rl_bind_key('\t', rl_insert);
 	/* Bind both ESC-TAB and ESC-ESC to the completion function */
@@ -524,6 +618,9 @@ setup_readline(void)
 	rl_completer_word_break_characters =
 		strdup(" \t\n`~!@#$%^&*()-=+[{]}\\|;:'\",<>/?");
 		/* All nonalphanums except '.' */
+#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
+	rl_completion_append_character ='\0';
+#endif
 
 	begidx = PyInt_FromLong(0L);
 	endidx = PyInt_FromLong(0L);
@@ -533,6 +630,11 @@ setup_readline(void)
 	 * inside this function.  Nothing we can do about it.
 	 */
 	rl_initialize();
+
+#ifdef SAVE_LOCALE
+	setlocale(LC_CTYPE, saved_locale); /* Restore locale */
+	free(saved_locale);
+#endif
 }
 
 
@@ -551,12 +653,18 @@ onintr(int sig)
 /* Wrapper around GNU readline that handles signals differently. */
 
 static char *
-call_readline(char *prompt)
+call_readline(FILE *sys_stdin, FILE *sys_stdout, char *prompt)
 {
 	size_t n;
 	char *p, *q;
 	PyOS_sighandler_t old_inthandler;
-	
+#ifdef SAVE_LOCALE
+	char *saved_locale = strdup(setlocale(LC_CTYPE, NULL));
+	if (!saved_locale)
+		Py_FatalError("not enough memory to save locale");
+	setlocale(LC_CTYPE, "");
+#endif
+
 	old_inthandler = PyOS_setsig(SIGINT, onintr);
 	if (setjmp(jbuf)) {
 #ifdef HAVE_SIGRELSE
@@ -567,6 +675,15 @@ call_readline(char *prompt)
 		return NULL;
 	}
 	rl_event_hook = PyOS_InputHook;
+
+	if (sys_stdin != rl_instream || sys_stdout != rl_outstream) {
+		rl_instream = sys_stdin;
+		rl_outstream = sys_stdout;
+#ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
+		rl_prep_terminal (1);
+#endif
+	}
+
 	p = readline(prompt);
 	PyOS_setsig(SIGINT, old_inthandler);
 
@@ -578,8 +695,23 @@ call_readline(char *prompt)
 		return p;
 	}
 	n = strlen(p);
-	if (n > 0)
-		add_history(p);
+	if (n > 0) {
+		char *line;
+		HISTORY_STATE *state = history_get_history_state();
+		if (state->length > 0)
+			line = history_get(state->length)->line;
+		else
+			line = "";
+		if (strcmp(p, line))
+			add_history(p);
+		/* the history docs don't say so, but the address of state
+		   changes each time history_get_history_state is called
+		   which makes me think it's freshly malloc'd memory...
+		   on the other hand, the address of the last line stays the
+		   same as long as history isn't extended, so it appears to
+		   be malloc'd but managed by the history package... */
+		free(state);
+	}
 	/* Copy the malloc'ed buffer into a PyMem_Malloc'ed one and
 	   release the original. */
 	q = p;
@@ -590,24 +722,27 @@ call_readline(char *prompt)
 		p[n+1] = '\0';
 	}
 	free(q);
+#ifdef SAVE_LOCALE
+	setlocale(LC_CTYPE, saved_locale); /* Restore locale */
+	free(saved_locale);
+#endif
 	return p;
 }
 
 
 /* Initialize the module */
 
-static char doc_module[] =
-"Importing this module enables command line editing using GNU readline.";
+PyDoc_STRVAR(doc_module,
+"Importing this module enables command line editing using GNU readline.");
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initreadline(void)
 {
 	PyObject *m;
 
 	m = Py_InitModule4("readline", readline_methods, doc_module,
 			   (PyObject *)NULL, PYTHON_API_VERSION);
-	if (isatty(fileno(stdin))) {
-		PyOS_ReadlineFunctionPointer = call_readline;
-		setup_readline();
-	}
+
+	PyOS_ReadlineFunctionPointer = call_readline;
+	setup_readline();
 }
