@@ -1,18 +1,21 @@
-
 /* strop module */
 
-static char strop_module__doc__[] =
-"Common string manipulations, optimized for speed.\n\
-\n\
-Always use \"import string\" rather than referencing\n\
-this module directly.";
-
 #include "Python.h"
-
 #include <ctype.h>
+
+PyDoc_STRVAR(strop_module__doc__,
+"Common string manipulations, optimized for speed.\n"
+"\n"
+"Always use \"import string\" rather than referencing\n"
+"this module directly.");
+
 /* XXX This file assumes that the <ctype.h> is*() functions
    XXX are defined for all 8-bit characters! */
 
+#define WARN if (PyErr_Warn(PyExc_DeprecationWarning, \
+		       "strop functions are obsolete; use string methods")) \
+	     return NULL
+
 /* The lstrip(), rstrip() and strip() functions are implemented
    in do_strip(), which uses an additional parameter to indicate what
    type of strip should occur. */
@@ -77,16 +80,16 @@ split_whitespace(char *s, int len, int maxsplit)
 }
 
 
-static char splitfields__doc__[] =
-"split(s [,sep [,maxsplit]]) -> list of strings\n\
-splitfields(s [,sep [,maxsplit]]) -> list of strings\n\
-\n\
-Return a list of the words in the string s, using sep as the\n\
-delimiter string.  If maxsplit is nonzero, splits into at most\n\
-maxsplit words.  If sep is not specified, any whitespace string\n\
-is a separator.  Maxsplit defaults to 0.\n\
-\n\
-(split and splitfields are synonymous)";
+PyDoc_STRVAR(splitfields__doc__,
+"split(s [,sep [,maxsplit]]) -> list of strings\n"
+"splitfields(s [,sep [,maxsplit]]) -> list of strings\n"
+"\n"
+"Return a list of the words in the string s, using sep as the\n"
+"delimiter string.  If maxsplit is nonzero, splits into at most\n"
+"maxsplit words.  If sep is not specified, any whitespace string\n"
+"is a separator.  Maxsplit defaults to 0.\n"
+"\n"
+"(split and splitfields are synonymous)");
 
 static PyObject *
 strop_splitfields(PyObject *self, PyObject *args)
@@ -96,6 +99,7 @@ strop_splitfields(PyObject *self, PyObject *args)
 	char *s, *sub;
 	PyObject *list, *item;
 
+	WARN;
 	sub = NULL;
 	n = 0;
 	splitcount = 0;
@@ -147,15 +151,15 @@ strop_splitfields(PyObject *self, PyObject *args)
 }
 
 
-static char joinfields__doc__[] =
-"join(list [,sep]) -> string\n\
-joinfields(list [,sep]) -> string\n\
-\n\
-Return a string composed of the words in list, with\n\
-intervening occurrences of sep.  Sep defaults to a single\n\
-space.\n\
-\n\
-(join and joinfields are synonymous)";
+PyDoc_STRVAR(joinfields__doc__,
+"join(list [,sep]) -> string\n"
+"joinfields(list [,sep]) -> string\n"
+"\n"
+"Return a string composed of the words in list, with\n"
+"intervening occurrences of sep.  Sep defaults to a single\n"
+"space.\n"
+"\n"
+"(join and joinfields are synonymous)");
 
 static PyObject *
 strop_joinfields(PyObject *self, PyObject *args)
@@ -168,6 +172,7 @@ strop_joinfields(PyObject *self, PyObject *args)
 	char* p = NULL;
 	intargfunc getitemfunc;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "O|t#:join", &seq, &sep, &seplen))
 		return NULL;
 	if (sep == NULL) {
@@ -210,10 +215,8 @@ strop_joinfields(PyObject *self, PyObject *args)
 			}
 			slen = PyString_GET_SIZE(item);
 			while (reslen + slen + seplen >= sz) {
-				if (_PyString_Resize(&res, sz * 2)) {
-					Py_DECREF(res);
+				if (_PyString_Resize(&res, sz * 2) < 0)
 					return NULL;
-				}
 				sz *= 2;
 				p = PyString_AsString(res) + reslen;
 			}
@@ -226,10 +229,7 @@ strop_joinfields(PyObject *self, PyObject *args)
 			p += slen;
 			reslen += slen;
 		}
-		if (_PyString_Resize(&res, reslen)) {
-			Py_DECREF(res);
-			res = NULL;
-		}
+		_PyString_Resize(&res, reslen);
 		return res;
 	}
 
@@ -252,8 +252,7 @@ strop_joinfields(PyObject *self, PyObject *args)
 		}
 		slen = PyString_GET_SIZE(item);
 		while (reslen + slen + seplen >= sz) {
-			if (_PyString_Resize(&res, sz * 2)) {
-				Py_DECREF(res);
+			if (_PyString_Resize(&res, sz * 2) < 0) {
 				Py_DECREF(item);
 				return NULL;
 			}
@@ -270,22 +269,19 @@ strop_joinfields(PyObject *self, PyObject *args)
 		reslen += slen;
 		Py_DECREF(item);
 	}
-	if (_PyString_Resize(&res, reslen)) {
-		Py_DECREF(res);
-		res = NULL;
-	}
+	_PyString_Resize(&res, reslen);
 	return res;
 }
 
 
-static char find__doc__[] =
-"find(s, sub [,start [,end]]) -> in\n\
-\n\
-Return the lowest index in s where substring sub is found,\n\
-such that sub is contained within s[start,end].  Optional\n\
-arguments start and end are interpreted as in slice notation.\n\
-\n\
-Return -1 on failure.";
+PyDoc_STRVAR(find__doc__,
+"find(s, sub [,start [,end]]) -> in\n"
+"\n"
+"Return the lowest index in s where substring sub is found,\n"
+"such that sub is contained within s[start,end].  Optional\n"
+"arguments start and end are interpreted as in slice notation.\n"
+"\n"
+"Return -1 on failure.");
 
 static PyObject *
 strop_find(PyObject *self, PyObject *args)
@@ -293,6 +289,7 @@ strop_find(PyObject *self, PyObject *args)
 	char *s, *sub;
 	int len, n, i = 0, last = INT_MAX;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "t#t#|ii:find", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 
@@ -320,14 +317,14 @@ strop_find(PyObject *self, PyObject *args)
 }
 
 
-static char rfind__doc__[] =
-"rfind(s, sub [,start [,end]]) -> int\n\
-\n\
-Return the highest index in s where substring sub is found,\n\
-such that sub is contained within s[start,end].  Optional\n\
-arguments start and end are interpreted as in slice notation.\n\
-\n\
-Return -1 on failure.";
+PyDoc_STRVAR(rfind__doc__,
+"rfind(s, sub [,start [,end]]) -> int\n"
+"\n"
+"Return the highest index in s where substring sub is found,\n"
+"such that sub is contained within s[start,end].  Optional\n"
+"arguments start and end are interpreted as in slice notation.\n"
+"\n"
+"Return -1 on failure.");
 
 static PyObject *
 strop_rfind(PyObject *self, PyObject *args)
@@ -336,6 +333,7 @@ strop_rfind(PyObject *self, PyObject *args)
 	int len, n, j;
 	int i = 0, last = INT_MAX;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "t#t#|ii:rfind", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 
@@ -369,7 +367,7 @@ do_strip(PyObject *args, int striptype)
 	int len, i, j;
 
 
-	if (!PyArg_Parse(args, "t#", &s, &len))
+	if (PyString_AsStringAndSize(args, &s, &len))
 		return NULL;
 
 	i = 0;
@@ -396,47 +394,50 @@ do_strip(PyObject *args, int striptype)
 }
 
 
-static char strip__doc__[] =
-"strip(s) -> string\n\
-\n\
-Return a copy of the string s with leading and trailing\n\
-whitespace removed.";
+PyDoc_STRVAR(strip__doc__,
+"strip(s) -> string\n"
+"\n"
+"Return a copy of the string s with leading and trailing\n"
+"whitespace removed.");
 
 static PyObject *
 strop_strip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, BOTHSTRIP);
 }
 
 
-static char lstrip__doc__[] =
-"lstrip(s) -> string\n\
-\n\
-Return a copy of the string s with leading whitespace removed.";
+PyDoc_STRVAR(lstrip__doc__,
+"lstrip(s) -> string\n"
+"\n"
+"Return a copy of the string s with leading whitespace removed.");
 
 static PyObject *
 strop_lstrip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, LEFTSTRIP);
 }
 
 
-static char rstrip__doc__[] =
-"rstrip(s) -> string\n\
-\n\
-Return a copy of the string s with trailing whitespace removed.";
+PyDoc_STRVAR(rstrip__doc__,
+"rstrip(s) -> string\n"
+"\n"
+"Return a copy of the string s with trailing whitespace removed.");
 
 static PyObject *
 strop_rstrip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, RIGHTSTRIP);
 }
 
 
-static char lower__doc__[] =
-"lower(s) -> string\n\
-\n\
-Return a copy of the string s converted to lowercase.";
+PyDoc_STRVAR(lower__doc__,
+"lower(s) -> string\n"
+"\n"
+"Return a copy of the string s converted to lowercase.");
 
 static PyObject *
 strop_lower(PyObject *self, PyObject *args)
@@ -446,7 +447,8 @@ strop_lower(PyObject *self, PyObject *args)
 	PyObject *new;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
 	new = PyString_FromStringAndSize(NULL, n);
 	if (new == NULL)
@@ -471,10 +473,10 @@ strop_lower(PyObject *self, PyObject *args)
 }
 
 
-static char upper__doc__[] =
-"upper(s) -> string\n\
-\n\
-Return a copy of the string s converted to uppercase.";
+PyDoc_STRVAR(upper__doc__,
+"upper(s) -> string\n"
+"\n"
+"Return a copy of the string s converted to uppercase.");
 
 static PyObject *
 strop_upper(PyObject *self, PyObject *args)
@@ -484,7 +486,8 @@ strop_upper(PyObject *self, PyObject *args)
 	PyObject *new;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
 	new = PyString_FromStringAndSize(NULL, n);
 	if (new == NULL)
@@ -509,11 +512,11 @@ strop_upper(PyObject *self, PyObject *args)
 }
 
 
-static char capitalize__doc__[] =
-"capitalize(s) -> string\n\
-\n\
-Return a copy of the string s with only its first character\n\
-capitalized.";
+PyDoc_STRVAR(capitalize__doc__,
+"capitalize(s) -> string\n"
+"\n"
+"Return a copy of the string s with only its first character\n"
+"capitalized.");
 
 static PyObject *
 strop_capitalize(PyObject *self, PyObject *args)
@@ -523,7 +526,8 @@ strop_capitalize(PyObject *self, PyObject *args)
 	PyObject *new;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
 	new = PyString_FromStringAndSize(NULL, n);
 	if (new == NULL)
@@ -557,13 +561,13 @@ strop_capitalize(PyObject *self, PyObject *args)
 }
 
 
-static char expandtabs__doc__[] =
-"expandtabs(string, [tabsize]) -> string\n\
-\n\
-Expand tabs in a string, i.e. replace them by one or more spaces,\n\
-depending on the current column and the given tab size (default 8).\n\
-The column number is reset to zero after each newline occurring in the\n\
-string.  This doesn't understand other non-printing characters.";
+PyDoc_STRVAR(expandtabs__doc__,
+"expandtabs(string, [tabsize]) -> string\n"
+"\n"
+"Expand tabs in a string, i.e. replace them by one or more spaces,\n"
+"depending on the current column and the given tab size (default 8).\n"
+"The column number is reset to zero after each newline occurring in the\n"
+"string.  This doesn't understand other non-printing characters.");
 
 static PyObject *
 strop_expandtabs(PyObject *self, PyObject *args)
@@ -572,12 +576,13 @@ strop_expandtabs(PyObject *self, PyObject *args)
 	char* e;
 	char* p;
 	char* q;
-	int i, j;
+	int i, j, old_j;
 	PyObject* out;
 	char* string;
 	int stringlen;
 	int tabsize = 8;
 
+	WARN;
 	/* Get arguments */
 	if (!PyArg_ParseTuple(args, "s#|i:expandtabs", &string, &stringlen, &tabsize))
 		return NULL;
@@ -588,12 +593,18 @@ strop_expandtabs(PyObject *self, PyObject *args)
 	}
 
 	/* First pass: determine size of output string */
-	i = j = 0; /* j: current column; i: total of previous lines */
+	i = j = old_j = 0; /* j: current column; i: total of previous lines */
 	e = string + stringlen;
 	for (p = string; p < e; p++) {
-		if (*p == '\t')
+		if (*p == '\t') {
 			j += tabsize - (j%tabsize);
-		else {
+			if (old_j > j) {
+				PyErr_SetString(PyExc_OverflowError,
+						"new string is too long");
+				return NULL;
+			}
+			old_j = j;
+		} else {
 			j++;
 			if (*p == '\n') {
 				i += j;
@@ -602,6 +613,11 @@ strop_expandtabs(PyObject *self, PyObject *args)
 		}
 	}
 
+	if ((i + j) < 0) {
+		PyErr_SetString(PyExc_OverflowError, "new string is too long");
+		return NULL;
+	}
+
 	/* Second pass: create output string and fill it */
 	out = PyString_FromStringAndSize(NULL, i+j);
 	if (out == NULL)
@@ -628,12 +644,12 @@ strop_expandtabs(PyObject *self, PyObject *args)
 }
 
 
-static char count__doc__[] =
-"count(s, sub[, start[, end]]) -> int\n\
-\n\
-Return the number of occurrences of substring sub in string\n\
-s[start:end].  Optional arguments start and end are\n\
-interpreted as in slice notation.";
+PyDoc_STRVAR(count__doc__,
+"count(s, sub[, start[, end]]) -> int\n"
+"\n"
+"Return the number of occurrences of substring sub in string\n"
+"s[start:end].  Optional arguments start and end are\n"
+"interpreted as in slice notation.");
 
 static PyObject *
 strop_count(PyObject *self, PyObject *args)
@@ -643,6 +659,7 @@ strop_count(PyObject *self, PyObject *args)
 	int i = 0, last = INT_MAX;
 	int m, r;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "t#t#|ii:count", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 	if (last > len)
@@ -672,11 +689,11 @@ strop_count(PyObject *self, PyObject *args)
 }
 
 
-static char swapcase__doc__[] =
-"swapcase(s) -> string\n\
-\n\
-Return a copy of the string s with upper case characters\n\
-converted to lowercase and vice versa.";
+PyDoc_STRVAR(swapcase__doc__,
+"swapcase(s) -> string\n"
+"\n"
+"Return a copy of the string s with upper case characters\n"
+"converted to lowercase and vice versa.");
 
 static PyObject *
 strop_swapcase(PyObject *self, PyObject *args)
@@ -686,7 +703,8 @@ strop_swapcase(PyObject *self, PyObject *args)
 	PyObject *new;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
 	new = PyString_FromStringAndSize(NULL, n);
 	if (new == NULL)
@@ -716,15 +734,15 @@ strop_swapcase(PyObject *self, PyObject *args)
 }
 
 
-static char atoi__doc__[] =
-"atoi(s [,base]) -> int\n\
-\n\
-Return the integer represented by the string s in the given\n\
-base, which defaults to 10.  The string s must consist of one\n\
-or more digits, possibly preceded by a sign.  If base is 0, it\n\
-is chosen from the leading characters of s, 0 for octal, 0x or\n\
-0X for hexadecimal.  If base is 16, a preceding 0x or 0X is\n\
-accepted.";
+PyDoc_STRVAR(atoi__doc__,
+"atoi(s [,base]) -> int\n"
+"\n"
+"Return the integer represented by the string s in the given\n"
+"base, which defaults to 10.  The string s must consist of one\n"
+"or more digits, possibly preceded by a sign.  If base is 0, it\n"
+"is chosen from the leading characters of s, 0 for octal, 0x or\n"
+"0X for hexadecimal.  If base is 16, a preceding 0x or 0X is\n"
+"accepted.");
 
 static PyObject *
 strop_atoi(PyObject *self, PyObject *args)
@@ -734,6 +752,7 @@ strop_atoi(PyObject *self, PyObject *args)
 	long x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s|i:atoi", &s, &base))
 		return NULL;
 
@@ -749,18 +768,20 @@ strop_atoi(PyObject *self, PyObject *args)
 		x = (long) PyOS_strtoul(s, &end, base);
 	else
 		x = PyOS_strtol(s, &end, base);
-	if (end == s || !isalnum(end[-1]))
+	if (end == s || !isalnum((int)end[-1]))
 		goto bad;
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
   bad:
-		sprintf(buffer, "invalid literal for atoi(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atoi(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
 	else if (errno != 0) {
-		sprintf(buffer, "atoi() literal too large: %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer), 
+			      "atoi() literal too large: %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
@@ -768,16 +789,16 @@ strop_atoi(PyObject *self, PyObject *args)
 }
 
 
-static char atol__doc__[] =
-"atol(s [,base]) -> long\n\
-\n\
-Return the long integer represented by the string s in the\n\
-given base, which defaults to 10.  The string s must consist\n\
-of one or more digits, possibly preceded by a sign.  If base\n\
-is 0, it is chosen from the leading characters of s, 0 for\n\
-octal, 0x or 0X for hexadecimal.  If base is 16, a preceding\n\
-0x or 0X is accepted.  A trailing L or l is not accepted,\n\
-unless base is 0.";
+PyDoc_STRVAR(atol__doc__,
+"atol(s [,base]) -> long\n"
+"\n"
+"Return the long integer represented by the string s in the\n"
+"given base, which defaults to 10.  The string s must consist\n"
+"of one or more digits, possibly preceded by a sign.  If base\n"
+"is 0, it is chosen from the leading characters of s, 0 for\n"
+"octal, 0x or 0X for hexadecimal.  If base is 16, a preceding\n"
+"0x or 0X is accepted.  A trailing L or l is not accepted,\n"
+"unless base is 0.");
 
 static PyObject *
 strop_atol(PyObject *self, PyObject *args)
@@ -787,6 +808,7 @@ strop_atol(PyObject *self, PyObject *args)
 	PyObject *x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s|i:atol", &s, &base))
 		return NULL;
 
@@ -809,7 +831,8 @@ strop_atol(PyObject *self, PyObject *args)
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
-		sprintf(buffer, "invalid literal for atol(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atol(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		Py_DECREF(x);
 		return NULL;
@@ -818,10 +841,10 @@ strop_atol(PyObject *self, PyObject *args)
 }
 
 
-static char atof__doc__[] =
-"atof(s) -> float\n\
-\n\
-Return the floating point number represented by the string s.";
+PyDoc_STRVAR(atof__doc__,
+"atof(s) -> float\n"
+"\n"
+"Return the floating point number represented by the string s.");
 
 static PyObject *
 strop_atof(PyObject *self, PyObject *args)
@@ -831,6 +854,7 @@ strop_atof(PyObject *self, PyObject *args)
 	double x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s:atof", &s))
 		return NULL;
 	while (*s && isspace(Py_CHARMASK(*s)))
@@ -846,12 +870,14 @@ strop_atof(PyObject *self, PyObject *args)
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
-		sprintf(buffer, "invalid literal for atof(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atof(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
 	else if (errno != 0) {
-		sprintf(buffer, "atof() literal too large: %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer), 
+			      "atof() literal too large: %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
@@ -859,12 +885,12 @@ strop_atof(PyObject *self, PyObject *args)
 }
 
 
-static char maketrans__doc__[] =
-"maketrans(frm, to) -> string\n\
-\n\
-Return a translation table (a string of 256 bytes long)\n\
-suitable for use in string.translate.  The strings frm and to\n\
-must be of the same length.";
+PyDoc_STRVAR(maketrans__doc__,
+"maketrans(frm, to) -> string\n"
+"\n"
+"Return a translation table (a string of 256 bytes long)\n"
+"suitable for use in string.translate.  The strings frm and to\n"
+"must be of the same length.");
 
 static PyObject *
 strop_maketrans(PyObject *self, PyObject *args)
@@ -895,13 +921,13 @@ strop_maketrans(PyObject *self, PyObject *args)
 }
 
 
-static char translate__doc__[] =
-"translate(s,table [,deletechars]) -> string\n\
-\n\
-Return a copy of the string s, where all characters occurring\n\
-in the optional argument deletechars are removed, and the\n\
-remaining characters have been mapped through the given\n\
-translation table, which must be a string of length 256.";
+PyDoc_STRVAR(translate__doc__,
+"translate(s,table [,deletechars]) -> string\n"
+"\n"
+"Return a copy of the string s, where all characters occurring\n"
+"in the optional argument deletechars are removed, and the\n"
+"remaining characters have been mapped through the given\n"
+"translation table, which must be a string of length 256.");
 
 static PyObject *
 strop_translate(PyObject *self, PyObject *args)
@@ -914,6 +940,7 @@ strop_translate(PyObject *self, PyObject *args)
 	PyObject *result;
 	int trans_table[256];
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "St#|t#:translate", &input_obj,
 			      &table1, &tablen, &del_table, &dellen))
 		return NULL;
@@ -964,8 +991,8 @@ strop_translate(PyObject *self, PyObject *args)
 		return input_obj;
 	}
 	/* Fix the size of the resulting string */
-	if (inlen > 0 &&_PyString_Resize(&result, output-output_start))
-		return NULL;
+	if (inlen > 0)
+		_PyString_Resize(&result, output - output_start);
 	return result;
 }
 
@@ -982,7 +1009,8 @@ strop_translate(PyObject *self, PyObject *args)
   found, or -1 if not found.  If len of PAT is greater than length of
   MEM, the function returns -1.
 */
-static int mymemfind(char *mem, int len, char *pat, int pat_len)
+static int 
+mymemfind(const char *mem, int len, const char *pat, int pat_len)
 {
 	register int ii;
 
@@ -1006,7 +1034,8 @@ static int mymemfind(char *mem, int len, char *pat, int pat_len)
    meaning mem=1111 and pat==11 returns 2.
            mem=11111 and pat==11 also return 2.
  */
-static int mymemcnt(char *mem, int len, char *pat, int pat_len)
+static int 
+mymemcnt(const char *mem, int len, const char *pat, int pat_len)
 {
 	register int offset = 0;
 	int nfound = 0;
@@ -1041,7 +1070,12 @@ static int mymemcnt(char *mem, int len, char *pat, int pat_len)
        the new string allocated locally, or
        NULL if an error occurred.
 */
-static char *mymemreplace(char *str, int len, char *pat, int pat_len, char *sub, int sub_len, int count, int *out_len)
+static char *
+mymemreplace(const char *str, int len,		/* input string */
+             const char *pat, int pat_len,	/* pattern string to find */
+             const char *sub, int sub_len,	/* substitution string */
+             int count,				/* number of replacements */
+	     int *out_len)
 {
 	char *out_s;
 	char *new_s;
@@ -1052,63 +1086,73 @@ static char *mymemreplace(char *str, int len, char *pat, int pat_len, char *sub,
 
 	/* find length of output string */
 	nfound = mymemcnt(str, len, pat, pat_len);
-	if (count > 0)
-		nfound = nfound > count ? count : nfound;
+	if (count < 0)
+		count = INT_MAX;
+	else if (nfound > count)
+		nfound = count;
 	if (nfound == 0)
 		goto return_same;
+
 	new_len = len + nfound*(sub_len - pat_len);
-
-	new_s = (char *)PyMem_MALLOC(new_len);
-	if (new_s == NULL) return NULL;
-
-	*out_len = new_len;
-	out_s = new_s;
-
-	while (len > 0) {
-		/* find index of next instance of pattern */
-		offset = mymemfind(str, len, pat, pat_len);
-		/* if not found,  break out of loop */
-		if (offset == -1) break;
-
-		/* copy non matching part of input string */
-		memcpy(new_s, str, offset); /* copy part of str before pat */
-		str += offset + pat_len; /* move str past pattern */
-		len -= offset + pat_len; /* reduce length of str remaining */
-
-		/* copy substitute into the output string */
-		new_s += offset; /* move new_s to dest for sub string */
-		memcpy(new_s, sub, sub_len); /* copy substring into new_s */
-		new_s += sub_len; /* offset new_s past sub string */
-
-		/* break when we've done count replacements */
-		if (--count == 0) break;
+	if (new_len == 0) {
+		/* Have to allocate something for the caller to free(). */
+		out_s = (char *)PyMem_MALLOC(1);
+		if (out_s == NULL)
+			return NULL;
+		out_s[0] = '\0';
 	}
-	/* copy any remaining values into output string */
-	if (len > 0)
-		memcpy(new_s, str, len);
+	else {
+		assert(new_len > 0);
+		new_s = (char *)PyMem_MALLOC(new_len);
+		if (new_s == NULL)
+			return NULL;
+		out_s = new_s;
+
+		for (; count > 0 && len > 0; --count) {
+			/* find index of next instance of pattern */
+			offset = mymemfind(str, len, pat, pat_len);
+			if (offset == -1)
+				break;
+
+			/* copy non matching part of input string */
+			memcpy(new_s, str, offset);
+			str += offset + pat_len;
+			len -= offset + pat_len;
+
+			/* copy substitute into the output string */
+			new_s += offset;
+			memcpy(new_s, sub, sub_len);
+			new_s += sub_len;
+		}
+		/* copy any remaining values into output string */
+		if (len > 0)
+			memcpy(new_s, str, len);
+	}
+	*out_len = new_len;
 	return out_s;
 
   return_same:
 	*out_len = -1;
-	return str;
+	return (char *)str; /* cast away const */
 }
 
 
-static char replace__doc__[] =
-"replace (str, old, new[, maxsplit]) -> string\n\
-\n\
-Return a copy of string str with all occurrences of substring\n\
-old replaced by new. If the optional argument maxsplit is\n\
-given, only the first maxsplit occurrences are replaced.";
+PyDoc_STRVAR(replace__doc__,
+"replace (str, old, new[, maxsplit]) -> string\n"
+"\n"
+"Return a copy of string str with all occurrences of substring\n"
+"old replaced by new. If the optional argument maxsplit is\n"
+"given, only the first maxsplit occurrences are replaced.");
 
 static PyObject *
 strop_replace(PyObject *self, PyObject *args)
 {
 	char *str, *pat,*sub,*new_s;
 	int len,pat_len,sub_len,out_len;
-	int count = 0;
+	int count = -1;
 	PyObject *new;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "t#t#t#|i:replace",
 			      &str, &len, &pat, &pat_len, &sub, &sub_len,
 			      &count))
@@ -1117,6 +1161,12 @@ strop_replace(PyObject *self, PyObject *args)
 		PyErr_SetString(PyExc_ValueError, "empty pattern string");
 		return NULL;
 	}
+	/* CAUTION:  strop treats a replace count of 0 as infinity, unlke
+	 * current (2.1) string.py and string methods.  Preserve this for
+	 * ... well, hard to say for what <wink>.
+	 */
+	if (count == 0)
+		count = -1;
 	new_s = mymemreplace(str,len,pat,pat_len,sub,sub_len,count,&out_len);
 	if (new_s == NULL) {
 		PyErr_NoMemory();
@@ -1139,61 +1189,39 @@ strop_replace(PyObject *self, PyObject *args)
 
 static PyMethodDef
 strop_methods[] = {
-	{"atof",	strop_atof, 
-	 METH_VARARGS, atof__doc__},
-	{"atoi",	strop_atoi, 
-	 METH_VARARGS, atoi__doc__},
-	{"atol",	strop_atol, 
-	 METH_VARARGS, atol__doc__},
-	{"capitalize",	strop_capitalize, 
-	 METH_OLDARGS, capitalize__doc__},
-	{"count",	strop_count, 
-	 METH_VARARGS, count__doc__},
-	{"expandtabs",	strop_expandtabs, 
-	 METH_VARARGS, expandtabs__doc__},
-	{"find",	strop_find, 
-	 METH_VARARGS, find__doc__},
-	{"join",	strop_joinfields, 
-	 METH_VARARGS, joinfields__doc__},
-	{"joinfields",	strop_joinfields, 
-	 METH_VARARGS, joinfields__doc__},
-	{"lstrip",	strop_lstrip, 
-	 METH_OLDARGS, lstrip__doc__},
-	{"lower",	strop_lower, 
-	 METH_OLDARGS, lower__doc__},
-	{"maketrans",	strop_maketrans, 
-	 METH_VARARGS, maketrans__doc__},
-	{"replace",	strop_replace, 
-	 METH_VARARGS, replace__doc__},
-	{"rfind",	strop_rfind, 
-	 METH_VARARGS, rfind__doc__},
-	{"rstrip",	strop_rstrip, 
-	 METH_OLDARGS, rstrip__doc__},
-	{"split",	strop_splitfields, 
-	 METH_VARARGS, splitfields__doc__},
-	{"splitfields",	strop_splitfields, 
-	 METH_VARARGS, splitfields__doc__},
-	{"strip",	strop_strip, 
-	 METH_OLDARGS, strip__doc__},
-	{"swapcase",	strop_swapcase, 
-	 METH_OLDARGS, swapcase__doc__},
-	{"translate",	strop_translate, 
-	 METH_VARARGS, translate__doc__},
-	{"upper",	strop_upper, 
-	 METH_OLDARGS, upper__doc__},
+	{"atof",	strop_atof,	   METH_VARARGS, atof__doc__},
+	{"atoi",	strop_atoi,	   METH_VARARGS, atoi__doc__},
+	{"atol",	strop_atol,	   METH_VARARGS, atol__doc__},
+	{"capitalize",	strop_capitalize,  METH_O,       capitalize__doc__},
+	{"count",	strop_count,	   METH_VARARGS, count__doc__},
+	{"expandtabs",	strop_expandtabs,  METH_VARARGS, expandtabs__doc__},
+	{"find",	strop_find,	   METH_VARARGS, find__doc__},
+	{"join",	strop_joinfields,  METH_VARARGS, joinfields__doc__},
+	{"joinfields",	strop_joinfields,  METH_VARARGS, joinfields__doc__},
+	{"lstrip",	strop_lstrip,	   METH_O,       lstrip__doc__},
+	{"lower",	strop_lower,	   METH_O,       lower__doc__},
+	{"maketrans",	strop_maketrans,   METH_VARARGS, maketrans__doc__},
+	{"replace",	strop_replace,	   METH_VARARGS, replace__doc__},
+	{"rfind",	strop_rfind,	   METH_VARARGS, rfind__doc__},
+	{"rstrip",	strop_rstrip,	   METH_O,       rstrip__doc__},
+	{"split",	strop_splitfields, METH_VARARGS, splitfields__doc__},
+	{"splitfields",	strop_splitfields, METH_VARARGS, splitfields__doc__},
+	{"strip",	strop_strip,	   METH_O,       strip__doc__},
+	{"swapcase",	strop_swapcase,    METH_O,       swapcase__doc__},
+	{"translate",	strop_translate,   METH_VARARGS, translate__doc__},
+	{"upper",	strop_upper,	   METH_O,       upper__doc__},
 	{NULL,		NULL}	/* sentinel */
 };
 
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initstrop(void)
 {
-	PyObject *m, *d, *s;
+	PyObject *m, *s;
 	char buf[256];
 	int c, n;
 	m = Py_InitModule4("strop", strop_methods, strop_module__doc__,
 			   (PyObject*)NULL, PYTHON_API_VERSION);
-	d = PyModule_GetDict(m);
 
 	/* Create 'whitespace' object */
 	n = 0;
@@ -1202,10 +1230,9 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "whitespace", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "whitespace", s);
+
 	/* Create 'lowercase' object */
 	n = 0;
 	for (c = 0; c < 256; c++) {
@@ -1213,10 +1240,8 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "lowercase", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "lowercase", s);
 
 	/* Create 'uppercase' object */
 	n = 0;
@@ -1225,8 +1250,6 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "uppercase", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "uppercase", s);
 }
